/* jpeg2html.c
Takes a directory and outputs suitable thumbnails and html to represent it
(C) 1998 N. Douglas
based on the Independent JPEG group's JPEG library
*/

#define RATIO 1/4

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "_fileop.h"
#include "god.h"

#include "jpeg-9a/cdjpeg.h"

static void Main_DoDirectory(char *rootpath);

static void atexithandler(void)
{
  printf("Debug intercept\n");
}

int main(int argc, char *args[])
{
  char rootpath[_MAX_PATH];

  atexit(atexithandler);
  if(argc<2)
  {
    fprintf(stderr,"Please enter a directory to translate\n");
    return 1;
  }
  strcpy(rootpath, args[1]);
  printf("Given '%s'\n",rootpath);

  /* First, does pic and thpic exist yet? */
  if(!_FileOp_Exists(_FileOp_Concantate(rootpath,"pic")))
  {
    _FileOp_Enum fileidx;
    BOOLEAN cont;
    /* Make the directories pic and thpic */
    printf("Directory not in correct format! Converting ...\n");
    _FileOp_CreateDir(_FileOp_Concantate(rootpath,"pic"));
    _FileOp_CreateDir(_FileOp_Concantate(rootpath,"thpic"));

    cont=_FileOp_BeginEnum(&fileidx,rootpath);
    while(cont)
    {
      char temp[_MAX_PATH];
      strcpy(temp,_FileOp_Concantate(rootpath,"pic"));
      strcpy(temp,_FileOp_Concantate(temp,_FileOp_GetLeaf(fileidx.path)));
      printf("Moving %s to %s ...\n",fileidx.path, temp);
      _FileOp_MoveFile(fileidx.path, temp);

      cont=_FileOp_NextEnum(&fileidx);
    }
    printf("\n");
  }

  /* Enumerate all files in pic and thumbnail them into thpic */
  printf("Beginning conversion\n");
  Main_DoDirectory(rootpath);
  return 0;
}

static void Main_DoDirectory(char *rootpath)
{
  char picpath[_MAX_PATH], thpicpath[_MAX_PATH];
  FILE *htmlout;
  _FileOp_Enum fileidx;
  BOOLEAN cont;

  strcpy(picpath, _FileOp_Concantate(rootpath, "pic"));
  strcpy(thpicpath, _FileOp_Concantate(rootpath, "thpic"));

  htmlout=fopen(_FileOp_Concantate(rootpath,"index.html"),"wt");
  if(htmlout==0)
  {
    fprintf(stderr,"Unable to open index file\n");
    exit(1);
  }
  fprintf(htmlout,"<html>\n<title>Pictures of %s</title>\n<!-- Generated by Jpeg2Html --!>\n<h1>Pictures of %s</h1>\n<p>\n",rootpath, rootpath);
  fprintf(htmlout,"<small>This index of thumbnails was generated by Jpeg2Html</small><p>\n");
  fprintf(htmlout,"<i>Click on a thumbnail to see the full size version</i><p>\n");

  cont=_FileOp_BeginEnum(&fileidx,picpath);
  while(cont)
  {
    FILE *in, *out;
    char outpath[_MAX_PATH];
    char temppath[_MAX_PATH];

    strcpy(temppath, _FileOp_Concantate(thpicpath, _FileOp_ChangeType(_FileOp_GetLeaf(fileidx.path),"ppm")));
    strcpy(outpath, _FileOp_Concantate(thpicpath, _FileOp_GetLeaf(fileidx.path)));

    /* First, see if the thumbnail already exists */
    if(_FileOp_Exists(outpath)) goto skipmake;

    /* Decompress and scale to PBM format */
    {
      struct jpeg_decompress_struct cinfo;
      struct jpeg_error_mgr jerr;
      djpeg_dest_ptr dest_mgr = NULL;

      cinfo.err=jpeg_std_error(&jerr);
      jpeg_create_decompress(&cinfo);

      in=fopen(fileidx.path,"rb");
      if(in==0)
      { 
        fprintf(stderr,"Unable to open input file\n");
        exit(1);
      }
      out=fopen(temppath, "wb");
      if(out==0)
      {
        fprintf(stderr,"Unable to open output temp file\n");
        exit(1);
      }
      jpeg_stdio_src(&cinfo,in);

      jpeg_read_header(&cinfo, TRUE);
      /* Set scaling */
      {
        JDIMENSION area;
        cinfo.scale_num=1;
        cinfo.scale_denom=4;
        area=cinfo.image_width*cinfo.image_height/(cinfo.scale_denom*cinfo.scale_denom);
        while(area>25*25)
        {
          cinfo.scale_denom++;
          area=cinfo.image_width*cinfo.image_height/(cinfo.scale_denom*cinfo.scale_denom);
        }
        while(area<50*50 && cinfo.scale_denom>1)
        {
          cinfo.scale_denom--;
          area=cinfo.image_width*cinfo.image_height/(cinfo.scale_denom*cinfo.scale_denom);
        }
      }
      printf("Decompressing and scaling down %s (%dx%d) to 1:%d ...\n",fileidx.path, cinfo.image_width, cinfo.image_height, cinfo.scale_denom);

      dest_mgr=jinit_write_ppm(&cinfo);
      dest_mgr->output_file=out;

      jpeg_start_decompress(&cinfo);

      (*dest_mgr->start_output)(&cinfo, dest_mgr);

      while (cinfo.output_scanline < cinfo.output_height)
      {
        JDIMENSION num_scanlines;
        num_scanlines = jpeg_read_scanlines(&cinfo, dest_mgr->buffer,
					  dest_mgr->buffer_height);
        (*dest_mgr->put_pixel_rows) (&cinfo, dest_mgr, num_scanlines);
      }

      (*dest_mgr->finish_output) (&cinfo, dest_mgr);
      (void) jpeg_finish_decompress(&cinfo);
      jpeg_destroy_decompress(&cinfo);

      fclose(in);
      fclose(out);
    }

    /* Compress temp PBM file back into jpeg */
    {
      struct jpeg_compress_struct cinfo;
      struct jpeg_error_mgr jerr;
      cjpeg_source_ptr src_mgr;
      
      cinfo.err=jpeg_std_error(&jerr);
      jpeg_create_compress(&cinfo);

      cinfo.in_color_space = JCS_RGB;
      jpeg_set_defaults(&cinfo);

      in=fopen(temppath,"rb");
      if(in==0)
      { 
        fprintf(stderr,"Unable to open input temp file\n");
        exit(1);
      }
      out=fopen(outpath, "wb");
      if(out==0)
      {
        fprintf(stderr,"Unable to open output file\n");
        exit(1);
      }
      printf("Compressing scaled down version of %s ...\n",fileidx.path);

      src_mgr=jinit_read_ppm(&cinfo);
      src_mgr->input_file=in;

      (*src_mgr->start_input)(&cinfo, src_mgr);

      jpeg_default_colorspace(&cinfo);

      jpeg_stdio_dest(&cinfo, out);

      jpeg_start_compress(&cinfo, TRUE);

      while (cinfo.next_scanline < cinfo.image_height)
      {
        JDIMENSION num_scanlines;
        num_scanlines = (*src_mgr->get_pixel_rows) (&cinfo, src_mgr);
        (void) jpeg_write_scanlines(&cinfo, src_mgr->buffer, num_scanlines);
      }

      (*src_mgr->finish_input) (&cinfo, src_mgr);
      jpeg_finish_compress(&cinfo);
      jpeg_destroy_compress(&cinfo);

      fclose(in);
      fclose(out);
    }

    remove(temppath);

skipmake:
    fprintf(htmlout,"<a href=\"pic/%s\"><img align=center alt=\"%s\" src=\"thpic/%s\"></a>\n",_FileOp_MakeUnix(_FileOp_GetLeaf(fileidx.path)),_FileOp_MakeUnix(_FileOp_GetLeaf(fileidx.path)),_FileOp_MakeUnix(_FileOp_GetLeaf(fileidx.path)));
    cont=_FileOp_NextEnum(&fileidx);
  }
  fprintf(htmlout,"</html>\n");
  fclose(htmlout);
}
